/*
 * Pidgin Authorization Greylisting
 * Copyright (C) 2011 Konrad Gräfe
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301, USA.
 */

#include "authorization-greylisting.h"

#include "authorization.h"
#include "request_info_hook.h"
#include "prefs.h"

#include <account.h>
#include <debug.h>
#include <gtkblist.h>
#include <minidialog.h>
#include <gtkutils.h>
#include <glib.h>

extern PurplePlugin *plugin;

typedef struct _greylist_entry {
	PurpleAccount* account;
	char *sender;
	guint timer;
	GTimeVal timestamp;
} GreylistEntry;

GList *greylist;

static gboolean greylist_entry_timedout_cb(gpointer data) {
	purple_debug_info(PLUGIN_STATIC_NAME, "Removing %s from greylist...\n", ((GreylistEntry *)data)->sender);

	greylist = g_list_remove(greylist, data);

	return FALSE; /* don't repeat timer */
}

static int request_authorization_cb (PurpleAccount* account, char *sender) {
	GtkWidget *mini_dialog;
	gboolean onTheList;
	GreylistEntry *entry;
	GList *cur;
	gchar *msg;
	gint minutes, seconds;
	GTimeVal timestamp;

	purple_debug_info(PLUGIN_STATIC_NAME, "account-authorization-requested()\n");

	if(purple_utf8_strcasecmp(purple_account_get_protocol_id(account), "prpl-icq")!=0) {
		return 0; /* business as usual */
	} else {
		/* okay. is he on our list? */
		onTheList = FALSE;
		cur = greylist;
		while(cur && !onTheList) {
			entry = (GreylistEntry *)cur->data;

			if(entry && entry->account == account && !purple_utf8_strcasecmp(entry->sender, sender)) {
				/* he is! let's prompt the user! */
				if(purple_prefs_get_bool(PLUGIN_PREFS_PREFIX "/show_time_difference")) {
					g_get_current_time(&timestamp);
					seconds = (timestamp.tv_sec - entry->timestamp.tv_sec) % 60;
					minutes = (timestamp.tv_sec - entry->timestamp.tv_sec) / 60;

					msg = g_strdup_printf(_("%s (%s) re-requested authorization after %d minutes and %d seconds."), sender, purple_account_get_protocol_name(account), minutes, seconds);
					mini_dialog = GTK_WIDGET(pidgin_mini_dialog_new(_("Authorization Greylisting"), msg, NULL));
					g_free(msg);

					pidgin_mini_dialog_add_button(PIDGIN_MINI_DIALOG(mini_dialog), _("OK"), NULL, NULL);

					pidgin_blist_add_alert(mini_dialog);
					
				}
				return 0;
			}
		}

		/* okay. he's not. let's greylist him! */

		if(purple_prefs_get_bool(PLUGIN_PREFS_PREFIX "/show_blocked_requests")) {
			msg = g_strdup_printf(_("Blocked authorization request from %s (%s)."), sender, purple_account_get_protocol_name(account));
			mini_dialog = GTK_WIDGET(pidgin_mini_dialog_new(_("Authorization Greylisting"), msg, NULL));
			g_free(msg);

			pidgin_mini_dialog_add_button(PIDGIN_MINI_DIALOG(mini_dialog), _("OK"), NULL, NULL);

			pidgin_blist_add_alert(mini_dialog);
		}

		entry = g_malloc(sizeof(GreylistEntry));
		entry->account = account;
		entry->sender = g_strdup(sender);
		g_get_current_time(&(entry->timestamp));

		greylist = g_list_append(greylist, entry);

		entry->timer = purple_timeout_add_seconds(60 * purple_prefs_get_int(PLUGIN_PREFS_PREFIX "/timeout"), greylist_entry_timedout_cb, entry);

		/* now Pidgin asks you for a reason..let's hook into this to answer it automatically */
		msg = prefs_get_deny_message();
		hook_into_request_info_once(msg);
		g_free(msg);

		return -1;
	}
}

void authorization_init(void) {
	purple_signal_connect(
		purple_accounts_get_handle(),
		"account-authorization-requested",
		plugin,
		PURPLE_CALLBACK(request_authorization_cb),
		NULL
	);

	greylist = NULL;


}


void authorization_uninit(void) {
}


